import _objectWithoutProperties from "@babel/runtime/helpers/esm/objectWithoutProperties";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
const _excluded = ["size", "itemRender", "buildOptionText", "selectComponentClass", "responsive"];
import { createVNode as _createVNode, createTextVNode as _createTextVNode } from "vue";
import { computed, toRef, defineComponent } from 'vue';
import LeftOutlined from "@ant-design/icons-vue/es/icons/LeftOutlined";
import RightOutlined from "@ant-design/icons-vue/es/icons/RightOutlined";
import DoubleLeftOutlined from "@ant-design/icons-vue/es/icons/DoubleLeftOutlined";
import DoubleRightOutlined from "@ant-design/icons-vue/es/icons/DoubleRightOutlined";
import VcSelect from '../select';
import MiniSelect from './MiniSelect';
import { useLocaleReceiver } from '../locale-provider/LocaleReceiver';
import VcPagination from '../vc-pagination';
import enUS from '../vc-pagination/locale/en_US';
import classNames from '../_util/classNames';
import useConfigInject from '../_util/hooks/useConfigInject';
import useBreakpoint from '../_util/hooks/useBreakpoint';
export const paginationProps = () => ({
  total: Number,
  defaultCurrent: Number,
  disabled: {
    type: Boolean,
    default: undefined
  },
  current: Number,
  defaultPageSize: Number,
  pageSize: Number,
  hideOnSinglePage: {
    type: Boolean,
    default: undefined
  },
  showSizeChanger: {
    type: Boolean,
    default: undefined
  },
  pageSizeOptions: Array,
  buildOptionText: Function,
  showQuickJumper: {
    type: [Boolean, Object],
    default: undefined
  },
  showTotal: Function,
  size: String,
  simple: {
    type: Boolean,
    default: undefined
  },
  locale: Object,
  prefixCls: String,
  selectPrefixCls: String,
  totalBoundaryShowSizeChanger: Number,
  selectComponentClass: String,
  itemRender: Function,
  role: String,
  responsive: Boolean,
  showLessItems: {
    type: Boolean,
    default: undefined
  },
  onChange: Function,
  onShowSizeChange: Function,
  'onUpdate:current': Function,
  'onUpdate:pageSize': Function
});
export const paginationConfig = () => _objectSpread(_objectSpread({}, paginationProps()), {}, {
  position: String
});
export default defineComponent({
  compatConfig: {
    MODE: 3
  },
  name: 'APagination',
  inheritAttrs: false,
  props: paginationProps(),
  // emits: ['change', 'showSizeChange', 'update:current', 'update:pageSize'],
  setup(props, _ref) {
    let {
      slots,
      attrs
    } = _ref;
    const {
      prefixCls,
      configProvider,
      direction
    } = useConfigInject('pagination', props);
    const selectPrefixCls = computed(() => configProvider.getPrefixCls('select', props.selectPrefixCls));
    const breakpoint = useBreakpoint();
    const [locale] = useLocaleReceiver('Pagination', enUS, toRef(props, 'locale'));
    const getIconsProps = pre => {
      const ellipsis = _createVNode("span", {
        "class": `${pre}-item-ellipsis`
      }, [_createTextVNode("\u2022\u2022\u2022")]);
      let prevIcon = _createVNode("button", {
        "class": `${pre}-item-link`,
        "type": "button",
        "tabindex": -1
      }, [_createVNode(LeftOutlined, null, null)]);
      let nextIcon = _createVNode("button", {
        "class": `${pre}-item-link`,
        "type": "button",
        "tabindex": -1
      }, [_createVNode(RightOutlined, null, null)]);
      let jumpPrevIcon = _createVNode("a", {
        "rel": "nofollow",
        "class": `${pre}-item-link`
      }, [_createVNode("div", {
        "class": `${pre}-item-container`
      }, [_createVNode(DoubleLeftOutlined, {
        "class": `${pre}-item-link-icon`
      }, null), ellipsis])]);
      let jumpNextIcon = _createVNode("a", {
        "rel": "nofollow",
        "class": `${pre}-item-link`
      }, [_createVNode("div", {
        "class": `${pre}-item-container`
      }, [_createVNode(DoubleRightOutlined, {
        "class": `${pre}-item-link-icon`
      }, null), ellipsis])]);
      // change arrows direction in right-to-left direction
      if (direction.value === 'rtl') {
        [prevIcon, nextIcon] = [nextIcon, prevIcon];
        [jumpPrevIcon, jumpNextIcon] = [jumpNextIcon, jumpPrevIcon];
      }
      return {
        prevIcon,
        nextIcon,
        jumpPrevIcon,
        jumpNextIcon
      };
    };
    return () => {
      var _breakpoint$value;
      const {
          size,
          itemRender = slots.itemRender,
          buildOptionText = slots.buildOptionText,
          selectComponentClass,
          responsive
        } = props,
        restProps = _objectWithoutProperties(props, _excluded);
      const isSmall = size === 'small' || !!((_breakpoint$value = breakpoint.value) !== null && _breakpoint$value !== void 0 && _breakpoint$value.xs && !size && responsive);
      const paginationProps = _objectSpread(_objectSpread(_objectSpread(_objectSpread({}, restProps), getIconsProps(prefixCls.value)), {}, {
        prefixCls: prefixCls.value,
        selectPrefixCls: selectPrefixCls.value,
        selectComponentClass: selectComponentClass || (isSmall ? MiniSelect : VcSelect),
        locale: locale.value,
        buildOptionText
      }, attrs), {}, {
        class: classNames({
          mini: isSmall,
          [`${prefixCls.value}-rtl`]: direction.value === 'rtl'
        }, attrs.class),
        itemRender
      });
      return _createVNode(VcPagination, paginationProps, null);
    };
  }
});