import { createVNode as _createVNode } from "vue";
import { defineComponent, computed } from 'vue';
import PropTypes from '../_util/vue-types';
import CheckCircleFilled from "@ant-design/icons-vue/es/icons/CheckCircleFilled";
import CloseCircleFilled from "@ant-design/icons-vue/es/icons/CloseCircleFilled";
import ExclamationCircleFilled from "@ant-design/icons-vue/es/icons/ExclamationCircleFilled";
import WarningFilled from "@ant-design/icons-vue/es/icons/WarningFilled";
import noFound from './noFound';
import serverError from './serverError';
import unauthorized from './unauthorized';
import useConfigInject from '../_util/hooks/useConfigInject';
import classNames from '../_util/classNames';
export const IconMap = {
  success: CheckCircleFilled,
  error: CloseCircleFilled,
  info: ExclamationCircleFilled,
  warning: WarningFilled
};
export const ExceptionMap = {
  '404': noFound,
  '500': serverError,
  '403': unauthorized
};
// ExceptionImageMap keys
const ExceptionStatus = Object.keys(ExceptionMap);
export const resultProps = () => ({
  prefixCls: String,
  icon: PropTypes.any,
  status: {
    type: [Number, String],
    default: 'info'
  },
  title: PropTypes.any,
  subTitle: PropTypes.any,
  extra: PropTypes.any
});
const renderIcon = (prefixCls, _ref) => {
  let {
    status,
    icon
  } = _ref;
  if (ExceptionStatus.includes(`${status}`)) {
    const SVGComponent = ExceptionMap[status];
    return _createVNode("div", {
      "class": `${prefixCls}-icon ${prefixCls}-image`
    }, [_createVNode(SVGComponent, null, null)]);
  }
  const IconComponent = IconMap[status];
  const iconNode = icon || _createVNode(IconComponent, null, null);
  return _createVNode("div", {
    "class": `${prefixCls}-icon`
  }, [iconNode]);
};
const renderExtra = (prefixCls, extra) => extra && _createVNode("div", {
  "class": `${prefixCls}-extra`
}, [extra]);
const Result = defineComponent({
  compatConfig: {
    MODE: 3
  },
  name: 'AResult',
  props: resultProps(),
  slots: Object,
  setup(props, _ref2) {
    let {
      slots
    } = _ref2;
    const {
      prefixCls,
      direction
    } = useConfigInject('result', props);
    const className = computed(() => classNames(prefixCls.value, `${prefixCls.value}-${props.status}`, {
      [`${prefixCls.value}-rtl`]: direction.value === 'rtl'
    }));
    return () => {
      var _slots$title, _slots$subTitle, _slots$icon, _slots$extra;
      const title = props.title ?? ((_slots$title = slots.title) === null || _slots$title === void 0 ? void 0 : _slots$title.call(slots));
      const subTitle = props.subTitle ?? ((_slots$subTitle = slots.subTitle) === null || _slots$subTitle === void 0 ? void 0 : _slots$subTitle.call(slots));
      const icon = props.icon ?? ((_slots$icon = slots.icon) === null || _slots$icon === void 0 ? void 0 : _slots$icon.call(slots));
      const extra = props.extra ?? ((_slots$extra = slots.extra) === null || _slots$extra === void 0 ? void 0 : _slots$extra.call(slots));
      const pre = prefixCls.value;
      return _createVNode("div", {
        "class": className.value
      }, [renderIcon(pre, {
        status: props.status,
        icon
      }), _createVNode("div", {
        "class": `${pre}-title`
      }, [title]), subTitle && _createVNode("div", {
        "class": `${pre}-subtitle`
      }, [subTitle]), renderExtra(pre, extra), slots.default && _createVNode("div", {
        "class": `${pre}-content`
      }, [slots.default()])]);
    };
  }
});
/* add resource */
Result.PRESENTED_IMAGE_403 = ExceptionMap[403];
Result.PRESENTED_IMAGE_404 = ExceptionMap[404];
Result.PRESENTED_IMAGE_500 = ExceptionMap[500];
/* istanbul ignore next */
Result.install = function (app) {
  app.component(Result.name, Result);
  return app;
};
export default Result;