import type { CSSProperties, VNodeTypes, PropType, ExtractPropTypes } from 'vue';
import type { CustomSlotsType, VueNode } from '../_util/type';
import type { TooltipPlacement } from '../tooltip/Tooltip';
import type { FocusEventHandler } from '../_util/EventInterface';
export type SliderValue = number | [number, number];
export interface SliderMarks {
    [key: number]: VueNode | {
        style: CSSProperties;
        label: any;
    };
}
interface HandleGeneratorInfo {
    value?: number;
    dragging?: boolean;
    index: number;
}
export interface SliderRange {
    draggableTrack?: boolean;
}
export type HandleGeneratorFn = (config: {
    tooltipPrefixCls?: string;
    prefixCls?: string;
    info: HandleGeneratorInfo;
}) => VNodeTypes;
type Value = [number, number] | number;
export declare const sliderProps: () => {
    id: StringConstructor;
    prefixCls: StringConstructor;
    tooltipPrefixCls: StringConstructor;
    range: {
        type: PropType<boolean | SliderRange>;
        default: any;
    };
    reverse: {
        type: BooleanConstructor;
        default: any;
    };
    min: NumberConstructor;
    max: NumberConstructor;
    step: {
        type: PropType<null | number>;
    };
    marks: {
        type: PropType<SliderMarks>;
    };
    dots: {
        type: BooleanConstructor;
        default: any;
    };
    value: {
        type: PropType<Value>;
    };
    defaultValue: {
        type: PropType<Value>;
    };
    included: {
        type: BooleanConstructor;
        default: any;
    };
    disabled: {
        type: BooleanConstructor;
        default: any;
    };
    vertical: {
        type: BooleanConstructor;
        default: any;
    };
    tipFormatter: {
        type: PropType<((value?: number) => any) | null>;
        default: () => (value: number) => string;
    };
    tooltipVisible: {
        type: BooleanConstructor;
        default: any;
    };
    tooltipPlacement: {
        type: PropType<TooltipPlacement>;
    };
    getTooltipPopupContainer: {
        type: PropType<(triggerNode: HTMLElement) => HTMLElement>;
    };
    autofocus: {
        type: BooleanConstructor;
        default: any;
    };
    handleStyle: {
        type: PropType<CSSProperties[] | CSSProperties>;
    };
    trackStyle: {
        type: PropType<CSSProperties[] | CSSProperties>;
    };
    onChange: {
        type: PropType<(value: Value) => void>;
    };
    onAfterChange: {
        type: PropType<(value: Value) => void>;
    };
    onFocus: {
        type: PropType<FocusEventHandler>;
    };
    onBlur: {
        type: PropType<FocusEventHandler>;
    };
    'onUpdate:value': {
        type: PropType<(value: Value) => void>;
    };
};
export type SliderProps = Partial<ExtractPropTypes<ReturnType<typeof sliderProps>>>;
export type Visibles = {
    [index: number]: boolean;
};
declare const _default: {
    new (...args: any[]): import("vue").CreateComponentPublicInstance<Readonly<ExtractPropTypes<{
        id: StringConstructor;
        prefixCls: StringConstructor;
        tooltipPrefixCls: StringConstructor;
        range: {
            type: PropType<boolean | SliderRange>;
            default: any;
        };
        reverse: {
            type: BooleanConstructor;
            default: any;
        };
        min: NumberConstructor;
        max: NumberConstructor;
        step: {
            type: PropType<null | number>;
        };
        marks: {
            type: PropType<SliderMarks>;
        };
        dots: {
            type: BooleanConstructor;
            default: any;
        };
        value: {
            type: PropType<Value>;
        };
        defaultValue: {
            type: PropType<Value>;
        };
        included: {
            type: BooleanConstructor;
            default: any;
        };
        disabled: {
            type: BooleanConstructor;
            default: any;
        };
        vertical: {
            type: BooleanConstructor;
            default: any;
        };
        tipFormatter: {
            type: PropType<((value?: number) => any) | null>;
            default: () => (value: number) => string;
        };
        tooltipVisible: {
            type: BooleanConstructor;
            default: any;
        };
        tooltipPlacement: {
            type: PropType<TooltipPlacement>;
        };
        getTooltipPopupContainer: {
            type: PropType<(triggerNode: HTMLElement) => HTMLElement>;
        };
        autofocus: {
            type: BooleanConstructor;
            default: any;
        };
        handleStyle: {
            type: PropType<CSSProperties[] | CSSProperties>;
        };
        trackStyle: {
            type: PropType<CSSProperties[] | CSSProperties>;
        };
        onChange: {
            type: PropType<(value: Value) => void>;
        };
        onAfterChange: {
            type: PropType<(value: Value) => void>;
        };
        onFocus: {
            type: PropType<FocusEventHandler>;
        };
        onBlur: {
            type: PropType<FocusEventHandler>;
        };
        'onUpdate:value': {
            type: PropType<(value: Value) => void>;
        };
    }>>, () => import("vue/jsx-runtime").JSX.Element, unknown, {}, {}, import("vue").ComponentOptionsMixin, import("vue").ComponentOptionsMixin, {}, import("vue").VNodeProps & import("vue").AllowedComponentProps & import("vue").ComponentCustomProps & Readonly<ExtractPropTypes<{
        id: StringConstructor;
        prefixCls: StringConstructor;
        tooltipPrefixCls: StringConstructor;
        range: {
            type: PropType<boolean | SliderRange>;
            default: any;
        };
        reverse: {
            type: BooleanConstructor;
            default: any;
        };
        min: NumberConstructor;
        max: NumberConstructor;
        step: {
            type: PropType<null | number>;
        };
        marks: {
            type: PropType<SliderMarks>;
        };
        dots: {
            type: BooleanConstructor;
            default: any;
        };
        value: {
            type: PropType<Value>;
        };
        defaultValue: {
            type: PropType<Value>;
        };
        included: {
            type: BooleanConstructor;
            default: any;
        };
        disabled: {
            type: BooleanConstructor;
            default: any;
        };
        vertical: {
            type: BooleanConstructor;
            default: any;
        };
        tipFormatter: {
            type: PropType<((value?: number) => any) | null>;
            default: () => (value: number) => string;
        };
        tooltipVisible: {
            type: BooleanConstructor;
            default: any;
        };
        tooltipPlacement: {
            type: PropType<TooltipPlacement>;
        };
        getTooltipPopupContainer: {
            type: PropType<(triggerNode: HTMLElement) => HTMLElement>;
        };
        autofocus: {
            type: BooleanConstructor;
            default: any;
        };
        handleStyle: {
            type: PropType<CSSProperties[] | CSSProperties>;
        };
        trackStyle: {
            type: PropType<CSSProperties[] | CSSProperties>;
        };
        onChange: {
            type: PropType<(value: Value) => void>;
        };
        onAfterChange: {
            type: PropType<(value: Value) => void>;
        };
        onFocus: {
            type: PropType<FocusEventHandler>;
        };
        onBlur: {
            type: PropType<FocusEventHandler>;
        };
        'onUpdate:value': {
            type: PropType<(value: Value) => void>;
        };
    }>>, {
        reverse: boolean;
        vertical: boolean;
        range: any;
        disabled: boolean;
        autofocus: boolean;
        dots: boolean;
        included: boolean;
        tipFormatter: (value?: number) => any;
        tooltipVisible: boolean;
    }, true, {}, CustomSlotsType<{
        mark?: any;
        default?: any;
    }>, {
        P: {};
        B: {};
        D: {};
        C: {};
        M: {};
        Defaults: {};
    }, Readonly<ExtractPropTypes<{
        id: StringConstructor;
        prefixCls: StringConstructor;
        tooltipPrefixCls: StringConstructor;
        range: {
            type: PropType<boolean | SliderRange>;
            default: any;
        };
        reverse: {
            type: BooleanConstructor;
            default: any;
        };
        min: NumberConstructor;
        max: NumberConstructor;
        step: {
            type: PropType<null | number>;
        };
        marks: {
            type: PropType<SliderMarks>;
        };
        dots: {
            type: BooleanConstructor;
            default: any;
        };
        value: {
            type: PropType<Value>;
        };
        defaultValue: {
            type: PropType<Value>;
        };
        included: {
            type: BooleanConstructor;
            default: any;
        };
        disabled: {
            type: BooleanConstructor;
            default: any;
        };
        vertical: {
            type: BooleanConstructor;
            default: any;
        };
        tipFormatter: {
            type: PropType<((value?: number) => any) | null>;
            default: () => (value: number) => string;
        };
        tooltipVisible: {
            type: BooleanConstructor;
            default: any;
        };
        tooltipPlacement: {
            type: PropType<TooltipPlacement>;
        };
        getTooltipPopupContainer: {
            type: PropType<(triggerNode: HTMLElement) => HTMLElement>;
        };
        autofocus: {
            type: BooleanConstructor;
            default: any;
        };
        handleStyle: {
            type: PropType<CSSProperties[] | CSSProperties>;
        };
        trackStyle: {
            type: PropType<CSSProperties[] | CSSProperties>;
        };
        onChange: {
            type: PropType<(value: Value) => void>;
        };
        onAfterChange: {
            type: PropType<(value: Value) => void>;
        };
        onFocus: {
            type: PropType<FocusEventHandler>;
        };
        onBlur: {
            type: PropType<FocusEventHandler>;
        };
        'onUpdate:value': {
            type: PropType<(value: Value) => void>;
        };
    }>>, () => import("vue/jsx-runtime").JSX.Element, {}, {}, {}, {
        reverse: boolean;
        vertical: boolean;
        range: any;
        disabled: boolean;
        autofocus: boolean;
        dots: boolean;
        included: boolean;
        tipFormatter: (value?: number) => any;
        tooltipVisible: boolean;
    }>;
    __isFragment?: never;
    __isTeleport?: never;
    __isSuspense?: never;
} & import("vue").ComponentOptionsBase<Readonly<ExtractPropTypes<{
    id: StringConstructor;
    prefixCls: StringConstructor;
    tooltipPrefixCls: StringConstructor;
    range: {
        type: PropType<boolean | SliderRange>;
        default: any;
    };
    reverse: {
        type: BooleanConstructor;
        default: any;
    };
    min: NumberConstructor;
    max: NumberConstructor;
    step: {
        type: PropType<null | number>;
    };
    marks: {
        type: PropType<SliderMarks>;
    };
    dots: {
        type: BooleanConstructor;
        default: any;
    };
    value: {
        type: PropType<Value>;
    };
    defaultValue: {
        type: PropType<Value>;
    };
    included: {
        type: BooleanConstructor;
        default: any;
    };
    disabled: {
        type: BooleanConstructor;
        default: any;
    };
    vertical: {
        type: BooleanConstructor;
        default: any;
    };
    tipFormatter: {
        type: PropType<((value?: number) => any) | null>;
        default: () => (value: number) => string;
    };
    tooltipVisible: {
        type: BooleanConstructor;
        default: any;
    };
    tooltipPlacement: {
        type: PropType<TooltipPlacement>;
    };
    getTooltipPopupContainer: {
        type: PropType<(triggerNode: HTMLElement) => HTMLElement>;
    };
    autofocus: {
        type: BooleanConstructor;
        default: any;
    };
    handleStyle: {
        type: PropType<CSSProperties[] | CSSProperties>;
    };
    trackStyle: {
        type: PropType<CSSProperties[] | CSSProperties>;
    };
    onChange: {
        type: PropType<(value: Value) => void>;
    };
    onAfterChange: {
        type: PropType<(value: Value) => void>;
    };
    onFocus: {
        type: PropType<FocusEventHandler>;
    };
    onBlur: {
        type: PropType<FocusEventHandler>;
    };
    'onUpdate:value': {
        type: PropType<(value: Value) => void>;
    };
}>>, () => import("vue/jsx-runtime").JSX.Element, unknown, {}, {}, import("vue").ComponentOptionsMixin, import("vue").ComponentOptionsMixin, {}, string, {
    reverse: boolean;
    vertical: boolean;
    range: any;
    disabled: boolean;
    autofocus: boolean;
    dots: boolean;
    included: boolean;
    tipFormatter: (value?: number) => any;
    tooltipVisible: boolean;
}, {}, string, CustomSlotsType<{
    mark?: any;
    default?: any;
}>> & (import("vue").VNodeProps & (import("vue").AllowedComponentProps & import("vue").ComponentCustomProps & import("vue").Plugin));
export default _default;
