import type { TableProps as RcTableProps } from '../vc-table/Table';
import type { SpinProps } from '../spin';
import type { TooltipProps } from '../tooltip';
import type { TableRowSelection, GetRowKey, ColumnType, ColumnsType, TableCurrentDataSource, SorterResult, GetPopupContainer, TablePaginationConfig, SortOrder, TableLocale, FilterValue } from './interface';
import type { SizeType } from '../config-provider';
import type { PropType } from 'vue';
import type { DefaultRecordType, RenderExpandIconProps } from '../vc-table/interface';
import type { CustomSlotsType } from '../_util/type';
export type { ColumnsType, TablePaginationConfig };
export interface TableProps<RecordType = DefaultRecordType> extends Omit<RcTableProps<RecordType>, 'transformColumns' | 'internalHooks' | 'internalRefs' | 'data' | 'columns' | 'scroll' | 'emptyText' | 'canExpandable' | 'onUpdateInternalRefs'> {
    dropdownPrefixCls?: string;
    dataSource?: RcTableProps<RecordType>['data'];
    columns?: ColumnsType<RecordType>;
    pagination?: false | TablePaginationConfig;
    loading?: boolean | SpinProps;
    size?: SizeType;
    bordered?: boolean;
    locale?: TableLocale;
    onChange?: (pagination: TablePaginationConfig, filters: Record<string, FilterValue | null>, sorter: SorterResult<RecordType> | SorterResult<RecordType>[], extra: TableCurrentDataSource<RecordType>) => void;
    rowSelection?: TableRowSelection<RecordType>;
    getPopupContainer?: GetPopupContainer;
    scroll?: RcTableProps<RecordType>['scroll'] & {
        scrollToFirstRowOnChange?: boolean;
    };
    sortDirections?: SortOrder[];
    showSorterTooltip?: boolean | TooltipProps;
}
export declare const tableProps: () => {
    prefixCls: {
        type: PropType<string>;
        default: any;
    };
    columns: {
        type: PropType<ColumnsType>;
        default: any;
    };
    rowKey: {
        type: PropType<TableProps["rowKey"]>;
        default: any;
    };
    tableLayout: {
        type: PropType<TableProps["tableLayout"]>;
        default: any;
    };
    rowClassName: {
        type: PropType<TableProps["rowClassName"]>;
        default: any;
    };
    title: {
        type: PropType<TableProps["title"]>;
        default: any;
    };
    footer: {
        type: PropType<TableProps["footer"]>;
        default: any;
    };
    id: {
        type: PropType<TableProps["id"]>;
        default: any;
    };
    showHeader: {
        type: PropType<TableProps["showHeader"]>;
        default: any;
    };
    components: {
        type: PropType<TableProps["components"]>;
        default: any;
    };
    customRow: {
        type: PropType<TableProps["customRow"]>;
        default: any;
    };
    customHeaderRow: {
        type: PropType<TableProps["customHeaderRow"]>;
        default: any;
    };
    direction: {
        type: PropType<TableProps["direction"]>;
        default: any;
    };
    expandFixed: {
        type: PropType<TableProps["expandFixed"]>;
        default: any;
    };
    expandColumnWidth: {
        type: PropType<TableProps["expandColumnWidth"]>;
        default: any;
    };
    expandedRowKeys: {
        type: PropType<TableProps["expandedRowKeys"]>;
        default: TableProps["expandedRowKeys"];
    };
    defaultExpandedRowKeys: {
        type: PropType<TableProps["defaultExpandedRowKeys"]>;
        default: TableProps["defaultExpandedRowKeys"];
    };
    expandedRowRender: {
        type: PropType<TableProps["expandedRowRender"]>;
        default: any;
    };
    expandRowByClick: {
        type: PropType<TableProps["expandRowByClick"]>;
        default: any;
    };
    expandIcon: {
        type: PropType<TableProps["expandIcon"]>;
        default: any;
    };
    onExpand: {
        type: PropType<TableProps["onExpand"]>;
        default: any;
    };
    onExpandedRowsChange: {
        type: PropType<TableProps["onExpandedRowsChange"]>;
        default: any;
    };
    'onUpdate:expandedRowKeys': {
        type: PropType<TableProps["onExpandedRowsChange"]>;
        default: any;
    };
    defaultExpandAllRows: {
        type: PropType<TableProps["defaultExpandAllRows"]>;
        default: any;
    };
    indentSize: {
        type: PropType<TableProps["indentSize"]>;
        default: any;
    };
    /** @deprecated Please use `EXPAND_COLUMN` in `columns` directly */
    expandIconColumnIndex: {
        type: PropType<TableProps["expandIconColumnIndex"]>;
        default: any;
    };
    showExpandColumn: {
        type: BooleanConstructor;
        default: any;
    };
    expandedRowClassName: {
        type: PropType<TableProps["expandedRowClassName"]>;
        default: any;
    };
    childrenColumnName: {
        type: PropType<TableProps["childrenColumnName"]>;
        default: any;
    };
    rowExpandable: {
        type: PropType<TableProps["rowExpandable"]>;
        default: any;
    };
    sticky: {
        type: PropType<TableProps["sticky"]>;
        default: any;
    };
    dropdownPrefixCls: StringConstructor;
    dataSource: {
        type: PropType<RcTableProps["data"]>;
        default: any;
    };
    pagination: {
        type: PropType<false | TablePaginationConfig>;
        default: any;
    };
    loading: {
        type: PropType<boolean | SpinProps>;
        default: any;
    };
    size: {
        type: PropType<SizeType>;
        default: any;
    };
    bordered: BooleanConstructor;
    locale: {
        type: PropType<TableLocale>;
        default: any;
    };
    onChange: {
        type: PropType<(pagination: TablePaginationConfig, filters: Record<string, FilterValue | null>, sorter: SorterResult | SorterResult[], extra: TableCurrentDataSource) => void>;
        default: any;
    };
    onResizeColumn: {
        type: PropType<(w: number, col: ColumnType) => void>;
        default: any;
    };
    rowSelection: {
        type: PropType<TableRowSelection>;
        default: any;
    };
    getPopupContainer: {
        type: PropType<GetPopupContainer>;
        default: any;
    };
    scroll: {
        type: PropType<RcTableProps["scroll"] & {
            scrollToFirstRowOnChange?: boolean;
        }>;
        default: any;
    };
    sortDirections: {
        type: PropType<SortOrder[]>;
        default: any;
    };
    showSorterTooltip: {
        type: PropType<boolean | TooltipProps>;
        default: boolean;
    };
    transformCellText: {
        type: PropType<TableProps["transformCellText"]>;
    };
};
declare const Table: import("vue").DefineComponent<{
    prefixCls: {
        type: PropType<string>;
        default: any;
    };
    columns: {
        type: PropType<ColumnsType>;
        default: any;
    };
    rowKey: {
        type: PropType<TableProps["rowKey"]>;
        default: any;
    };
    tableLayout: {
        type: PropType<TableProps["tableLayout"]>;
        default: any;
    };
    rowClassName: {
        type: PropType<TableProps["rowClassName"]>;
        default: any;
    };
    title: {
        type: PropType<TableProps["title"]>;
        default: any;
    };
    footer: {
        type: PropType<TableProps["footer"]>;
        default: any;
    };
    id: {
        type: PropType<TableProps["id"]>;
        default: any;
    };
    showHeader: {
        type: PropType<TableProps["showHeader"]>;
        default: any;
    };
    components: {
        type: PropType<TableProps["components"]>;
        default: any;
    };
    customRow: {
        type: PropType<TableProps["customRow"]>;
        default: any;
    };
    customHeaderRow: {
        type: PropType<TableProps["customHeaderRow"]>;
        default: any;
    };
    direction: {
        type: PropType<TableProps["direction"]>;
        default: any;
    };
    expandFixed: {
        type: PropType<TableProps["expandFixed"]>;
        default: any;
    };
    expandColumnWidth: {
        type: PropType<TableProps["expandColumnWidth"]>;
        default: any;
    };
    expandedRowKeys: {
        type: PropType<TableProps["expandedRowKeys"]>;
        default: TableProps["expandedRowKeys"];
    };
    defaultExpandedRowKeys: {
        type: PropType<TableProps["defaultExpandedRowKeys"]>;
        default: TableProps["defaultExpandedRowKeys"];
    };
    expandedRowRender: {
        type: PropType<TableProps["expandedRowRender"]>;
        default: any;
    };
    expandRowByClick: {
        type: PropType<TableProps["expandRowByClick"]>;
        default: any;
    };
    expandIcon: {
        type: PropType<TableProps["expandIcon"]>;
        default: any;
    };
    onExpand: {
        type: PropType<TableProps["onExpand"]>;
        default: any;
    };
    onExpandedRowsChange: {
        type: PropType<TableProps["onExpandedRowsChange"]>;
        default: any;
    };
    'onUpdate:expandedRowKeys': {
        type: PropType<TableProps["onExpandedRowsChange"]>;
        default: any;
    };
    defaultExpandAllRows: {
        type: PropType<TableProps["defaultExpandAllRows"]>;
        default: any;
    };
    indentSize: {
        type: PropType<TableProps["indentSize"]>;
        default: any;
    };
    /** @deprecated Please use `EXPAND_COLUMN` in `columns` directly */
    expandIconColumnIndex: {
        type: PropType<TableProps["expandIconColumnIndex"]>;
        default: any;
    };
    showExpandColumn: {
        type: BooleanConstructor;
        default: any;
    };
    expandedRowClassName: {
        type: PropType<TableProps["expandedRowClassName"]>;
        default: any;
    };
    childrenColumnName: {
        type: PropType<TableProps["childrenColumnName"]>;
        default: any;
    };
    rowExpandable: {
        type: PropType<TableProps["rowExpandable"]>;
        default: any;
    };
    sticky: {
        type: PropType<TableProps["sticky"]>;
        default: any;
    };
    dropdownPrefixCls: StringConstructor;
    dataSource: {
        type: PropType<RcTableProps["data"]>;
        default: any;
    };
    pagination: {
        type: PropType<false | TablePaginationConfig>;
        default: any;
    };
    loading: {
        type: PropType<boolean | SpinProps>;
        default: any;
    };
    size: {
        type: PropType<SizeType>;
        default: any;
    };
    bordered: BooleanConstructor;
    locale: {
        type: PropType<TableLocale>;
        default: any;
    };
    onChange: {
        type: PropType<(pagination: TablePaginationConfig, filters: Record<string, FilterValue | null>, sorter: SorterResult | SorterResult[], extra: TableCurrentDataSource) => void>;
        default: any;
    };
    onResizeColumn: {
        type: PropType<(w: number, col: ColumnType) => void>;
        default: any;
    };
    rowSelection: {
        type: PropType<TableRowSelection>;
        default: any;
    };
    getPopupContainer: {
        type: PropType<GetPopupContainer>;
        default: any;
    };
    scroll: {
        type: PropType<RcTableProps["scroll"] & {
            scrollToFirstRowOnChange?: boolean;
        }>;
        default: any;
    };
    sortDirections: {
        type: PropType<SortOrder[]>;
        default: any;
    };
    showSorterTooltip: {
        type: PropType<boolean | TooltipProps>;
        default: boolean;
    };
    transformCellText: {
        type: PropType<TableProps["transformCellText"]>;
    };
}, () => import("vue/jsx-runtime").JSX.Element, unknown, {}, {}, import("vue").ComponentOptionsMixin, import("vue").ComponentOptionsMixin, {}, string, import("vue").VNodeProps & import("vue").AllowedComponentProps & import("vue").ComponentCustomProps, Readonly<import("vue").ExtractPropTypes<{
    prefixCls: {
        type: PropType<string>;
        default: any;
    };
    columns: {
        type: PropType<ColumnsType>;
        default: any;
    };
    rowKey: {
        type: PropType<TableProps["rowKey"]>;
        default: any;
    };
    tableLayout: {
        type: PropType<TableProps["tableLayout"]>;
        default: any;
    };
    rowClassName: {
        type: PropType<TableProps["rowClassName"]>;
        default: any;
    };
    title: {
        type: PropType<TableProps["title"]>;
        default: any;
    };
    footer: {
        type: PropType<TableProps["footer"]>;
        default: any;
    };
    id: {
        type: PropType<TableProps["id"]>;
        default: any;
    };
    showHeader: {
        type: PropType<TableProps["showHeader"]>;
        default: any;
    };
    components: {
        type: PropType<TableProps["components"]>;
        default: any;
    };
    customRow: {
        type: PropType<TableProps["customRow"]>;
        default: any;
    };
    customHeaderRow: {
        type: PropType<TableProps["customHeaderRow"]>;
        default: any;
    };
    direction: {
        type: PropType<TableProps["direction"]>;
        default: any;
    };
    expandFixed: {
        type: PropType<TableProps["expandFixed"]>;
        default: any;
    };
    expandColumnWidth: {
        type: PropType<TableProps["expandColumnWidth"]>;
        default: any;
    };
    expandedRowKeys: {
        type: PropType<TableProps["expandedRowKeys"]>;
        default: TableProps["expandedRowKeys"];
    };
    defaultExpandedRowKeys: {
        type: PropType<TableProps["defaultExpandedRowKeys"]>;
        default: TableProps["defaultExpandedRowKeys"];
    };
    expandedRowRender: {
        type: PropType<TableProps["expandedRowRender"]>;
        default: any;
    };
    expandRowByClick: {
        type: PropType<TableProps["expandRowByClick"]>;
        default: any;
    };
    expandIcon: {
        type: PropType<TableProps["expandIcon"]>;
        default: any;
    };
    onExpand: {
        type: PropType<TableProps["onExpand"]>;
        default: any;
    };
    onExpandedRowsChange: {
        type: PropType<TableProps["onExpandedRowsChange"]>;
        default: any;
    };
    'onUpdate:expandedRowKeys': {
        type: PropType<TableProps["onExpandedRowsChange"]>;
        default: any;
    };
    defaultExpandAllRows: {
        type: PropType<TableProps["defaultExpandAllRows"]>;
        default: any;
    };
    indentSize: {
        type: PropType<TableProps["indentSize"]>;
        default: any;
    };
    /** @deprecated Please use `EXPAND_COLUMN` in `columns` directly */
    expandIconColumnIndex: {
        type: PropType<TableProps["expandIconColumnIndex"]>;
        default: any;
    };
    showExpandColumn: {
        type: BooleanConstructor;
        default: any;
    };
    expandedRowClassName: {
        type: PropType<TableProps["expandedRowClassName"]>;
        default: any;
    };
    childrenColumnName: {
        type: PropType<TableProps["childrenColumnName"]>;
        default: any;
    };
    rowExpandable: {
        type: PropType<TableProps["rowExpandable"]>;
        default: any;
    };
    sticky: {
        type: PropType<TableProps["sticky"]>;
        default: any;
    };
    dropdownPrefixCls: StringConstructor;
    dataSource: {
        type: PropType<RcTableProps["data"]>;
        default: any;
    };
    pagination: {
        type: PropType<false | TablePaginationConfig>;
        default: any;
    };
    loading: {
        type: PropType<boolean | SpinProps>;
        default: any;
    };
    size: {
        type: PropType<SizeType>;
        default: any;
    };
    bordered: BooleanConstructor;
    locale: {
        type: PropType<TableLocale>;
        default: any;
    };
    onChange: {
        type: PropType<(pagination: TablePaginationConfig, filters: Record<string, FilterValue | null>, sorter: SorterResult | SorterResult[], extra: TableCurrentDataSource) => void>;
        default: any;
    };
    onResizeColumn: {
        type: PropType<(w: number, col: ColumnType) => void>;
        default: any;
    };
    rowSelection: {
        type: PropType<TableRowSelection>;
        default: any;
    };
    getPopupContainer: {
        type: PropType<GetPopupContainer>;
        default: any;
    };
    scroll: {
        type: PropType<RcTableProps["scroll"] & {
            scrollToFirstRowOnChange?: boolean;
        }>;
        default: any;
    };
    sortDirections: {
        type: PropType<SortOrder[]>;
        default: any;
    };
    showSorterTooltip: {
        type: PropType<boolean | TooltipProps>;
        default: boolean;
    };
    transformCellText: {
        type: PropType<TableProps["transformCellText"]>;
    };
}>>, {
    onChange: (pagination: TablePaginationConfig, filters: Record<string, FilterValue | null>, sorter: SorterResult | SorterResult[], extra: TableCurrentDataSource) => void;
    prefixCls: string;
    footer: import("../vc-table/interface").PanelRender<any>;
    title: import("../vc-table/interface").PanelRender<any>;
    scroll: any;
    size: SizeType;
    sticky: any;
    rowClassName: string | import("../vc-table/interface").RowClassName<any>;
    locale: any;
    components: any;
    getPopupContainer: GetPopupContainer;
    direction: "ltr" | "rtl";
    tableLayout: import("../vc-table/interface").TableLayout;
    columns: ColumnsType;
    id: string;
    loading: any;
    onExpand: (expanded: boolean, record: any) => void;
    bordered: boolean;
    pagination: any;
    expandIcon: import("../vc-table/interface").RenderExpandIcon<any>;
    expandedRowKeys: import("../vc-table/interface").Key[];
    defaultExpandedRowKeys: import("../vc-table/interface").Key[];
    expandedRowRender: import("../vc-table/interface").ExpandedRowRender<any>;
    expandRowByClick: boolean;
    onExpandedRowsChange: (expandedKeys: import("../vc-table/interface").Key[]) => void;
    defaultExpandAllRows: boolean;
    indentSize: number;
    expandIconColumnIndex: number;
    showExpandColumn: boolean;
    expandedRowClassName: import("../vc-table/interface").RowClassName<any>;
    childrenColumnName: string;
    rowExpandable: (record: any) => boolean;
    onResizeColumn: (w: number, col: ColumnType) => void;
    customHeaderRow: import("../vc-table/interface").GetComponentProps<import("../vc-table/interface").ColumnType<any>[]>;
    customRow: import("../vc-table/interface").GetComponentProps<any>;
    rowKey: string | GetRowKey<any>;
    expandColumnWidth: number;
    expandFixed: boolean | "left" | "right";
    showHeader: boolean;
    'onUpdate:expandedRowKeys': (expandedKeys: import("../vc-table/interface").Key[]) => void;
    sortDirections: SortOrder[];
    showSorterTooltip: boolean;
    dataSource: any[];
    rowSelection: any;
}, CustomSlotsType<{
    emptyText?: any;
    expandIcon?: RenderExpandIconProps<any>;
    title?: any;
    footer?: any;
    summary?: any;
    expandedRowRender?: any;
    bodyCell?: {
        text: any;
        value: any;
        record: Record<string, any>;
        index: number;
        column: ColumnType;
    };
    headerCell?: {
        title: any;
        column: ColumnType;
    };
    customFilterIcon?: any;
    customFilterDropdown?: any;
    default: any;
}>>;
export default Table;
