import type { CSSProperties, ExtractPropTypes, PropType } from 'vue';
import type { CustomSlotsType, VueNode } from '../_util/type';
import type { TransferListBodyProps } from './ListBody';
import type { PaginationType } from './interface';
export type { TransferListProps } from './list';
export type { TransferOperationProps } from './operation';
export type { TransferSearchProps } from './search';
export type TransferDirection = 'left' | 'right';
export interface RenderResultObject {
    label: VueNode;
    value: string;
}
export type RenderResult = VueNode | RenderResultObject | string | null;
export interface TransferItem {
    key?: string;
    title?: string;
    description?: string;
    disabled?: boolean;
    [name: string]: any;
}
export type KeyWise<T> = T & {
    key: string;
};
export type KeyWiseTransferItem = KeyWise<TransferItem>;
type TransferRender<RecordType> = (item: RecordType) => RenderResult;
export interface ListStyle {
    direction: TransferDirection;
}
export type SelectAllLabel = VueNode | ((info: {
    selectedCount: number;
    totalCount: number;
}) => VueNode);
export interface TransferLocale {
    titles: VueNode[];
    notFoundContent?: VueNode;
    searchPlaceholder: string;
    itemUnit: string;
    itemsUnit: string;
    remove: string;
    selectAll: string;
    selectCurrent: string;
    selectInvert: string;
    removeAll: string;
    removeCurrent: string;
}
export declare const transferProps: () => {
    id: StringConstructor;
    prefixCls: StringConstructor;
    dataSource: {
        type: PropType<TransferItem[]>;
        default: any[];
    };
    disabled: {
        type: BooleanConstructor;
        default: any;
    };
    targetKeys: {
        type: PropType<string[]>;
        default: any;
    };
    selectedKeys: {
        type: PropType<string[]>;
        default: any;
    };
    render: {
        type: PropType<TransferRender<TransferItem>>;
    };
    listStyle: {
        type: PropType<((style: ListStyle) => CSSProperties) | CSSProperties>;
        default: () => {};
    };
    operationStyle: {
        type: PropType<CSSProperties>;
        default: CSSProperties;
    };
    titles: {
        type: PropType<string[]>;
    };
    operations: {
        type: PropType<string[]>;
    };
    showSearch: {
        type: BooleanConstructor;
        default: boolean;
    };
    filterOption: {
        type: PropType<(inputValue: string, item: TransferItem) => boolean>;
    };
    searchPlaceholder: StringConstructor;
    notFoundContent: import("vue-types").VueTypeValidableDef<any>;
    locale: {
        type: PropType<Partial<TransferLocale>>;
        default: () => {};
    };
    rowKey: {
        type: PropType<(record: TransferItem) => string>;
    };
    showSelectAll: {
        type: BooleanConstructor;
        default: any;
    };
    selectAllLabels: {
        type: PropType<SelectAllLabel[]>;
    };
    children: {
        type: PropType<(props: TransferListBodyProps) => VueNode>;
    };
    oneWay: {
        type: BooleanConstructor;
        default: any;
    };
    pagination: {
        type: PropType<PaginationType>;
        default: any;
    };
    onChange: PropType<(targetKeys: string[], direction: TransferDirection, moveKeys: string[]) => void>;
    onSelectChange: PropType<(sourceSelectedKeys: string[], targetSelectedKeys: string[]) => void>;
    onSearch: PropType<(direction: TransferDirection, value: string) => void>;
    onScroll: PropType<(direction: TransferDirection, e: UIEvent) => void>;
    'onUpdate:targetKeys': PropType<(keys: string[]) => void>;
    'onUpdate:selectedKeys': PropType<(keys: string[]) => void>;
};
export type TransferProps = Partial<ExtractPropTypes<ReturnType<typeof transferProps>>>;
declare const _default: {
    new (...args: any[]): import("vue").CreateComponentPublicInstance<Readonly<ExtractPropTypes<{
        id: StringConstructor;
        prefixCls: StringConstructor;
        dataSource: {
            type: PropType<TransferItem[]>;
            default: any[];
        };
        disabled: {
            type: BooleanConstructor;
            default: any;
        };
        targetKeys: {
            type: PropType<string[]>;
            default: any;
        };
        selectedKeys: {
            type: PropType<string[]>;
            default: any;
        };
        render: {
            type: PropType<TransferRender<TransferItem>>;
        };
        listStyle: {
            type: PropType<((style: ListStyle) => CSSProperties) | CSSProperties>;
            default: () => {};
        };
        operationStyle: {
            type: PropType<CSSProperties>;
            default: CSSProperties;
        };
        titles: {
            type: PropType<string[]>;
        };
        operations: {
            type: PropType<string[]>;
        };
        showSearch: {
            type: BooleanConstructor;
            default: boolean;
        };
        filterOption: {
            type: PropType<(inputValue: string, item: TransferItem) => boolean>;
        };
        searchPlaceholder: StringConstructor;
        notFoundContent: import("vue-types").VueTypeValidableDef<any>;
        locale: {
            type: PropType<Partial<TransferLocale>>;
            default: () => {};
        };
        rowKey: {
            type: PropType<(record: TransferItem) => string>;
        };
        showSelectAll: {
            type: BooleanConstructor;
            default: any;
        };
        selectAllLabels: {
            type: PropType<SelectAllLabel[]>;
        };
        children: {
            type: PropType<(props: TransferListBodyProps) => VueNode>;
        };
        oneWay: {
            type: BooleanConstructor;
            default: any;
        };
        pagination: {
            type: PropType<PaginationType>;
            default: any;
        };
        onChange: PropType<(targetKeys: string[], direction: TransferDirection, moveKeys: string[]) => void>;
        onSelectChange: PropType<(sourceSelectedKeys: string[], targetSelectedKeys: string[]) => void>;
        onSearch: PropType<(direction: TransferDirection, value: string) => void>;
        onScroll: PropType<(direction: TransferDirection, e: UIEvent) => void>;
        'onUpdate:targetKeys': PropType<(keys: string[]) => void>;
        'onUpdate:selectedKeys': PropType<(keys: string[]) => void>;
    }>>, () => import("vue/jsx-runtime").JSX.Element, unknown, {}, {}, import("vue").ComponentOptionsMixin, import("vue").ComponentOptionsMixin, {}, import("vue").VNodeProps & import("vue").AllowedComponentProps & import("vue").ComponentCustomProps & Readonly<ExtractPropTypes<{
        id: StringConstructor;
        prefixCls: StringConstructor;
        dataSource: {
            type: PropType<TransferItem[]>;
            default: any[];
        };
        disabled: {
            type: BooleanConstructor;
            default: any;
        };
        targetKeys: {
            type: PropType<string[]>;
            default: any;
        };
        selectedKeys: {
            type: PropType<string[]>;
            default: any;
        };
        render: {
            type: PropType<TransferRender<TransferItem>>;
        };
        listStyle: {
            type: PropType<((style: ListStyle) => CSSProperties) | CSSProperties>;
            default: () => {};
        };
        operationStyle: {
            type: PropType<CSSProperties>;
            default: CSSProperties;
        };
        titles: {
            type: PropType<string[]>;
        };
        operations: {
            type: PropType<string[]>;
        };
        showSearch: {
            type: BooleanConstructor;
            default: boolean;
        };
        filterOption: {
            type: PropType<(inputValue: string, item: TransferItem) => boolean>;
        };
        searchPlaceholder: StringConstructor;
        notFoundContent: import("vue-types").VueTypeValidableDef<any>;
        locale: {
            type: PropType<Partial<TransferLocale>>;
            default: () => {};
        };
        rowKey: {
            type: PropType<(record: TransferItem) => string>;
        };
        showSelectAll: {
            type: BooleanConstructor;
            default: any;
        };
        selectAllLabels: {
            type: PropType<SelectAllLabel[]>;
        };
        children: {
            type: PropType<(props: TransferListBodyProps) => VueNode>;
        };
        oneWay: {
            type: BooleanConstructor;
            default: any;
        };
        pagination: {
            type: PropType<PaginationType>;
            default: any;
        };
        onChange: PropType<(targetKeys: string[], direction: TransferDirection, moveKeys: string[]) => void>;
        onSelectChange: PropType<(sourceSelectedKeys: string[], targetSelectedKeys: string[]) => void>;
        onSearch: PropType<(direction: TransferDirection, value: string) => void>;
        onScroll: PropType<(direction: TransferDirection, e: UIEvent) => void>;
        'onUpdate:targetKeys': PropType<(keys: string[]) => void>;
        'onUpdate:selectedKeys': PropType<(keys: string[]) => void>;
    }>>, {
        disabled: boolean;
        locale: {};
        listStyle: CSSProperties | ((style: ListStyle) => CSSProperties);
        selectedKeys: string[];
        showSearch: boolean;
        pagination: any;
        dataSource: TransferItem[];
        showSelectAll: boolean;
        oneWay: boolean;
        targetKeys: string[];
        operationStyle: CSSProperties;
    }, true, {}, CustomSlotsType<{
        leftTitle?: any;
        rightTitle?: any;
        children?: any;
        render?: TransferItem;
        notFoundContent?: any;
        leftSelectAllLabel?: any;
        rightSelectAllLabel?: any;
        footer?: any;
        default?: any;
    }>, {
        P: {};
        B: {};
        D: {};
        C: {};
        M: {};
        Defaults: {};
    }, Readonly<ExtractPropTypes<{
        id: StringConstructor;
        prefixCls: StringConstructor;
        dataSource: {
            type: PropType<TransferItem[]>;
            default: any[];
        };
        disabled: {
            type: BooleanConstructor;
            default: any;
        };
        targetKeys: {
            type: PropType<string[]>;
            default: any;
        };
        selectedKeys: {
            type: PropType<string[]>;
            default: any;
        };
        render: {
            type: PropType<TransferRender<TransferItem>>;
        };
        listStyle: {
            type: PropType<((style: ListStyle) => CSSProperties) | CSSProperties>;
            default: () => {};
        };
        operationStyle: {
            type: PropType<CSSProperties>;
            default: CSSProperties;
        };
        titles: {
            type: PropType<string[]>;
        };
        operations: {
            type: PropType<string[]>;
        };
        showSearch: {
            type: BooleanConstructor;
            default: boolean;
        };
        filterOption: {
            type: PropType<(inputValue: string, item: TransferItem) => boolean>;
        };
        searchPlaceholder: StringConstructor;
        notFoundContent: import("vue-types").VueTypeValidableDef<any>;
        locale: {
            type: PropType<Partial<TransferLocale>>;
            default: () => {};
        };
        rowKey: {
            type: PropType<(record: TransferItem) => string>;
        };
        showSelectAll: {
            type: BooleanConstructor;
            default: any;
        };
        selectAllLabels: {
            type: PropType<SelectAllLabel[]>;
        };
        children: {
            type: PropType<(props: TransferListBodyProps) => VueNode>;
        };
        oneWay: {
            type: BooleanConstructor;
            default: any;
        };
        pagination: {
            type: PropType<PaginationType>;
            default: any;
        };
        onChange: PropType<(targetKeys: string[], direction: TransferDirection, moveKeys: string[]) => void>;
        onSelectChange: PropType<(sourceSelectedKeys: string[], targetSelectedKeys: string[]) => void>;
        onSearch: PropType<(direction: TransferDirection, value: string) => void>;
        onScroll: PropType<(direction: TransferDirection, e: UIEvent) => void>;
        'onUpdate:targetKeys': PropType<(keys: string[]) => void>;
        'onUpdate:selectedKeys': PropType<(keys: string[]) => void>;
    }>>, () => import("vue/jsx-runtime").JSX.Element, {}, {}, {}, {
        disabled: boolean;
        locale: {};
        listStyle: CSSProperties | ((style: ListStyle) => CSSProperties);
        selectedKeys: string[];
        showSearch: boolean;
        pagination: any;
        dataSource: TransferItem[];
        showSelectAll: boolean;
        oneWay: boolean;
        targetKeys: string[];
        operationStyle: CSSProperties;
    }>;
    __isFragment?: never;
    __isTeleport?: never;
    __isSuspense?: never;
} & import("vue").ComponentOptionsBase<Readonly<ExtractPropTypes<{
    id: StringConstructor;
    prefixCls: StringConstructor;
    dataSource: {
        type: PropType<TransferItem[]>;
        default: any[];
    };
    disabled: {
        type: BooleanConstructor;
        default: any;
    };
    targetKeys: {
        type: PropType<string[]>;
        default: any;
    };
    selectedKeys: {
        type: PropType<string[]>;
        default: any;
    };
    render: {
        type: PropType<TransferRender<TransferItem>>;
    };
    listStyle: {
        type: PropType<((style: ListStyle) => CSSProperties) | CSSProperties>;
        default: () => {};
    };
    operationStyle: {
        type: PropType<CSSProperties>;
        default: CSSProperties;
    };
    titles: {
        type: PropType<string[]>;
    };
    operations: {
        type: PropType<string[]>;
    };
    showSearch: {
        type: BooleanConstructor;
        default: boolean;
    };
    filterOption: {
        type: PropType<(inputValue: string, item: TransferItem) => boolean>;
    };
    searchPlaceholder: StringConstructor;
    notFoundContent: import("vue-types").VueTypeValidableDef<any>;
    locale: {
        type: PropType<Partial<TransferLocale>>;
        default: () => {};
    };
    rowKey: {
        type: PropType<(record: TransferItem) => string>;
    };
    showSelectAll: {
        type: BooleanConstructor;
        default: any;
    };
    selectAllLabels: {
        type: PropType<SelectAllLabel[]>;
    };
    children: {
        type: PropType<(props: TransferListBodyProps) => VueNode>;
    };
    oneWay: {
        type: BooleanConstructor;
        default: any;
    };
    pagination: {
        type: PropType<PaginationType>;
        default: any;
    };
    onChange: PropType<(targetKeys: string[], direction: TransferDirection, moveKeys: string[]) => void>;
    onSelectChange: PropType<(sourceSelectedKeys: string[], targetSelectedKeys: string[]) => void>;
    onSearch: PropType<(direction: TransferDirection, value: string) => void>;
    onScroll: PropType<(direction: TransferDirection, e: UIEvent) => void>;
    'onUpdate:targetKeys': PropType<(keys: string[]) => void>;
    'onUpdate:selectedKeys': PropType<(keys: string[]) => void>;
}>>, () => import("vue/jsx-runtime").JSX.Element, unknown, {}, {}, import("vue").ComponentOptionsMixin, import("vue").ComponentOptionsMixin, {}, string, {
    disabled: boolean;
    locale: {};
    listStyle: CSSProperties | ((style: ListStyle) => CSSProperties);
    selectedKeys: string[];
    showSearch: boolean;
    pagination: any;
    dataSource: TransferItem[];
    showSelectAll: boolean;
    oneWay: boolean;
    targetKeys: string[];
    operationStyle: CSSProperties;
}, {}, string, CustomSlotsType<{
    leftTitle?: any;
    rightTitle?: any;
    children?: any;
    render?: TransferItem;
    notFoundContent?: any;
    leftSelectAllLabel?: any;
    rightSelectAllLabel?: any;
    footer?: any;
    default?: any;
}>> & (import("vue").VNodeProps & (import("vue").AllowedComponentProps & import("vue").ComponentCustomProps & import("vue").Plugin));
export default _default;
