import { createVNode as _createVNode } from "vue";
import Menu, { Item as MenuItem } from '../../menu';
import { onBeforeUnmount, defineComponent, inject, ref } from 'vue';
import MentionsContextKey from './MentionsContext';
import Spin from '../../spin';
function noop() {}
export default defineComponent({
  compatConfig: {
    MODE: 3
  },
  name: 'DropdownMenu',
  props: {
    prefixCls: String,
    options: {
      type: Array,
      default: () => []
    }
  },
  setup(props, _ref) {
    let {
      slots
    } = _ref;
    const {
      activeIndex,
      setActiveIndex,
      selectOption,
      onFocus = noop,
      loading
    } = inject(MentionsContextKey, {
      activeIndex: ref(),
      loading: ref(false)
    });
    let timeoutId;
    const onMousedown = e => {
      clearTimeout(timeoutId);
      timeoutId = setTimeout(() => {
        onFocus(e);
      });
    };
    onBeforeUnmount(() => {
      clearTimeout(timeoutId);
    });
    return () => {
      var _slots$notFoundConten;
      const {
        prefixCls,
        options
      } = props;
      const activeOption = options[activeIndex.value] || {};
      return _createVNode(Menu, {
        "prefixCls": `${prefixCls}-menu`,
        "activeKey": activeOption.value,
        "onSelect": _ref2 => {
          let {
            key
          } = _ref2;
          const option = options.find(_ref3 => {
            let {
              value
            } = _ref3;
            return value === key;
          });
          selectOption(option);
        },
        "onMousedown": onMousedown
      }, {
        default: () => [!loading.value && options.map((option, index) => {
          var _slots$option;
          const {
            value,
            disabled,
            label = option.value
          } = option;
          return _createVNode(MenuItem, {
            "key": value,
            "disabled": disabled,
            "onMouseenter": () => {
              setActiveIndex(index);
            }
          }, {
            default: () => [((_slots$option = slots.option) === null || _slots$option === void 0 ? void 0 : _slots$option.call(slots, option)) ?? (typeof label === 'function' ? label({
              value,
              disabled
            }) : label)]
          });
        }), !loading.value && options.length === 0 ? _createVNode(MenuItem, {
          "key": "notFoundContent",
          "disabled": true
        }, {
          default: () => [(_slots$notFoundConten = slots.notFoundContent) === null || _slots$notFoundConten === void 0 ? void 0 : _slots$notFoundConten.call(slots)]
        }) : null, loading.value && _createVNode(MenuItem, {
          "key": "loading",
          "disabled": true
        }, {
          default: () => [_createVNode(Spin, {
            "size": "small"
          }, null)]
        })]
      });
    };
  }
});