"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _vue = require("vue");
var _Event = _interopRequireDefault(require("./css-animation/Event"));
var _raf = _interopRequireDefault(require("./raf"));
var _propsUtil = require("./props-util");
var _useConfigInject = _interopRequireDefault(require("./hooks/useConfigInject"));
let styleForPesudo;
// Where el is the DOM element you'd like to test for visibility
function isHidden(element) {
  if (process.env.NODE_ENV === 'test') {
    return false;
  }
  return !element || element.offsetParent === null;
}
function isNotGrey(color) {
  // eslint-disable-next-line no-useless-escape
  const match = (color || '').match(/rgba?\((\d*), (\d*), (\d*)(, [\.\d]*)?\)/);
  if (match && match[1] && match[2] && match[3]) {
    return !(match[1] === match[2] && match[2] === match[3]);
  }
  return true;
}
var _default = exports.default = (0, _vue.defineComponent)({
  compatConfig: {
    MODE: 3
  },
  name: 'Wave',
  props: {
    insertExtraNode: Boolean,
    disabled: Boolean
  },
  setup(props, _ref) {
    let {
      slots,
      expose
    } = _ref;
    const instance = (0, _vue.getCurrentInstance)();
    const {
      csp,
      prefixCls
    } = (0, _useConfigInject.default)('', props);
    expose({
      csp
    });
    let eventIns = null;
    let clickWaveTimeoutId = null;
    let animationStartId = null;
    let animationStart = false;
    let extraNode = null;
    let isUnmounted = false;
    const onTransitionStart = e => {
      if (isUnmounted) return;
      const node = (0, _propsUtil.findDOMNode)(instance);
      if (!e || e.target !== node) {
        return;
      }
      if (!animationStart) {
        resetEffect(node);
      }
    };
    const onTransitionEnd = e => {
      if (!e || e.animationName !== 'fadeEffect') {
        return;
      }
      resetEffect(e.target);
    };
    const getAttributeName = () => {
      const {
        insertExtraNode
      } = props;
      return insertExtraNode ? `${prefixCls.value}-click-animating` : `${prefixCls.value}-click-animating-without-extra-node`;
    };
    const onClick = (node, waveColor) => {
      const {
        insertExtraNode,
        disabled
      } = props;
      if (disabled || !node || isHidden(node) || node.className.indexOf('-leave') >= 0) {
        return;
      }
      extraNode = document.createElement('div');
      extraNode.className = `${prefixCls.value}-click-animating-node`;
      const attributeName = getAttributeName();
      node.removeAttribute(attributeName);
      node.setAttribute(attributeName, 'true');
      // Not white or transparent or grey
      styleForPesudo = styleForPesudo || document.createElement('style');
      if (waveColor && waveColor !== '#ffffff' && waveColor !== 'rgb(255, 255, 255)' && isNotGrey(waveColor) && !/rgba\(\d*, \d*, \d*, 0\)/.test(waveColor) &&
      // any transparent rgba color
      waveColor !== 'transparent') {
        var _csp$value;
        // Add nonce if CSP exist
        if ((_csp$value = csp.value) !== null && _csp$value !== void 0 && _csp$value.nonce) {
          styleForPesudo.nonce = csp.value.nonce;
        }
        extraNode.style.borderColor = waveColor;
        styleForPesudo.innerHTML = `
        [${prefixCls.value}-click-animating-without-extra-node='true']::after, .${prefixCls.value}-click-animating-node {
          --antd-wave-shadow-color: ${waveColor};
        }`;
        if (!document.body.contains(styleForPesudo)) {
          document.body.appendChild(styleForPesudo);
        }
      }
      if (insertExtraNode) {
        node.appendChild(extraNode);
      }
      _Event.default.addStartEventListener(node, onTransitionStart);
      _Event.default.addEndEventListener(node, onTransitionEnd);
    };
    const resetEffect = node => {
      if (!node || node === extraNode || !(node instanceof Element)) {
        return;
      }
      const {
        insertExtraNode
      } = props;
      const attributeName = getAttributeName();
      node.setAttribute(attributeName, 'false'); // edge has bug on `removeAttribute` #14466
      if (styleForPesudo) {
        styleForPesudo.innerHTML = '';
      }
      if (insertExtraNode && extraNode && node.contains(extraNode)) {
        node.removeChild(extraNode);
      }
      _Event.default.removeStartEventListener(node, onTransitionStart);
      _Event.default.removeEndEventListener(node, onTransitionEnd);
    };
    const bindAnimationEvent = node => {
      if (!node || !node.getAttribute || node.getAttribute('disabled') || node.className.indexOf('disabled') >= 0) {
        return;
      }
      const newClick = e => {
        // Fix radio button click twice
        if (e.target.tagName === 'INPUT' || isHidden(e.target)) {
          return;
        }
        resetEffect(node);
        // Get wave color from target
        const waveColor = getComputedStyle(node).getPropertyValue('border-top-color') ||
        // Firefox Compatible
        getComputedStyle(node).getPropertyValue('border-color') || getComputedStyle(node).getPropertyValue('background-color');
        clickWaveTimeoutId = setTimeout(() => onClick(node, waveColor), 0);
        _raf.default.cancel(animationStartId);
        animationStart = true;
        // Render to trigger transition event cost 3 frames. Let's delay 10 frames to reset this.
        animationStartId = (0, _raf.default)(() => {
          animationStart = false;
        }, 10);
      };
      node.addEventListener('click', newClick, true);
      return {
        cancel: () => {
          node.removeEventListener('click', newClick, true);
        }
      };
    };
    (0, _vue.onMounted)(() => {
      (0, _vue.nextTick)(() => {
        const node = (0, _propsUtil.findDOMNode)(instance);
        if (node.nodeType !== 1) {
          return;
        }
        eventIns = bindAnimationEvent(node);
      });
    });
    (0, _vue.onBeforeUnmount)(() => {
      if (eventIns) {
        eventIns.cancel();
      }
      clearTimeout(clickWaveTimeoutId);
      isUnmounted = true;
    });
    return () => {
      var _slots$default;
      return (_slots$default = slots.default) === null || _slots$default === void 0 ? void 0 : _slots$default.call(slots)[0];
    };
  }
});