"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.backTopProps = void 0;
var _vue = require("vue");
var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));
var _VerticalAlignTopOutlined = _interopRequireDefault(require("@ant-design/icons-vue/lib/icons/VerticalAlignTopOutlined"));
var _addEventListener = _interopRequireDefault(require("../vc-util/Dom/addEventListener"));
var _getScroll = _interopRequireDefault(require("../_util/getScroll"));
var _transition = require("../_util/transition");
var _scrollTo = _interopRequireDefault(require("../_util/scrollTo"));
var _type = require("../_util/type");
var _throttleByAnimationFrame = _interopRequireDefault(require("../_util/throttleByAnimationFrame"));
var _useConfigInject = _interopRequireDefault(require("../_util/hooks/useConfigInject"));
const backTopProps = () => ({
  visibilityHeight: {
    type: Number,
    default: 400
  },
  duration: {
    type: Number,
    default: 450
  },
  target: Function,
  prefixCls: String,
  onClick: Function
  // visible: { type: Boolean, default: undefined }, // Only for test. Don't use it.
});
exports.backTopProps = backTopProps;
const BackTop = (0, _vue.defineComponent)({
  compatConfig: {
    MODE: 3
  },
  name: 'ABackTop',
  inheritAttrs: false,
  props: backTopProps(),
  // emits: ['click'],
  setup(props, _ref) {
    let {
      slots,
      attrs,
      emit
    } = _ref;
    const {
      prefixCls,
      direction
    } = (0, _useConfigInject.default)('back-top', props);
    const domRef = (0, _vue.ref)();
    const state = (0, _vue.reactive)({
      visible: false,
      scrollEvent: null
    });
    const getDefaultTarget = () => domRef.value && domRef.value.ownerDocument ? domRef.value.ownerDocument : window;
    const scrollToTop = e => {
      const {
        target = getDefaultTarget,
        duration
      } = props;
      (0, _scrollTo.default)(0, {
        getContainer: target,
        duration
      });
      emit('click', e);
    };
    const handleScroll = (0, _throttleByAnimationFrame.default)(e => {
      const {
        visibilityHeight
      } = props;
      const scrollTop = (0, _getScroll.default)(e.target, true);
      state.visible = scrollTop > visibilityHeight;
    });
    const bindScrollEvent = () => {
      const {
        target
      } = props;
      const getTarget = target || getDefaultTarget;
      const container = getTarget();
      state.scrollEvent = (0, _addEventListener.default)(container, 'scroll', e => {
        handleScroll(e);
      });
      handleScroll({
        target: container
      });
    };
    const scrollRemove = () => {
      if (state.scrollEvent) {
        state.scrollEvent.remove();
      }
      handleScroll.cancel();
    };
    (0, _vue.watch)(() => props.target, () => {
      scrollRemove();
      (0, _vue.nextTick)(() => {
        bindScrollEvent();
      });
    });
    (0, _vue.onMounted)(() => {
      (0, _vue.nextTick)(() => {
        bindScrollEvent();
      });
    });
    (0, _vue.onActivated)(() => {
      (0, _vue.nextTick)(() => {
        bindScrollEvent();
      });
    });
    (0, _vue.onDeactivated)(() => {
      scrollRemove();
    });
    (0, _vue.onBeforeUnmount)(() => {
      scrollRemove();
    });
    return () => {
      var _slots$default;
      const defaultElement = (0, _vue.createVNode)("div", {
        "class": `${prefixCls.value}-content`
      }, [(0, _vue.createVNode)("div", {
        "class": `${prefixCls.value}-icon`
      }, [(0, _vue.createVNode)(_VerticalAlignTopOutlined.default, null, null)])]);
      const divProps = (0, _objectSpread2.default)((0, _objectSpread2.default)({}, attrs), {}, {
        onClick: scrollToTop,
        class: {
          [`${prefixCls.value}`]: true,
          [`${attrs.class}`]: attrs.class,
          [`${prefixCls.value}-rtl`]: direction.value === 'rtl'
        }
      });
      const transitionProps = (0, _transition.getTransitionProps)('fade');
      return (0, _vue.createVNode)(_transition.Transition, transitionProps, {
        default: () => [(0, _vue.withDirectives)((0, _vue.createVNode)("div", (0, _objectSpread2.default)((0, _objectSpread2.default)({}, divProps), {}, {
          "ref": domRef
        }), [((_slots$default = slots.default) === null || _slots$default === void 0 ? void 0 : _slots$default.call(slots)) || defaultElement]), [[_vue.vShow, state.visible]])]
      });
    };
  }
});
var _default = exports.default = (0, _type.withInstall)(BackTop);