"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.pageHeaderProps = exports.default = void 0;
var _vue = require("vue");
var _vueTypes = _interopRequireDefault(require("../_util/vue-types"));
var _propsUtil = require("../_util/props-util");
var _ArrowLeftOutlined = _interopRequireDefault(require("@ant-design/icons-vue/lib/icons/ArrowLeftOutlined"));
var _ArrowRightOutlined = _interopRequireDefault(require("@ant-design/icons-vue/lib/icons/ArrowRightOutlined"));
var _breadcrumb = _interopRequireDefault(require("../breadcrumb"));
var _avatar = _interopRequireDefault(require("../avatar"));
var _transButton = _interopRequireDefault(require("../_util/transButton"));
var _LocaleReceiver = _interopRequireDefault(require("../locale-provider/LocaleReceiver"));
var _type = require("../_util/type");
var _useConfigInject = _interopRequireDefault(require("../_util/hooks/useConfigInject"));
var _classNames = _interopRequireDefault(require("../_util/classNames"));
var _vcResizeObserver = _interopRequireDefault(require("../vc-resize-observer"));
var _useDestroyed = _interopRequireDefault(require("../_util/hooks/useDestroyed"));
const pageHeaderProps = () => ({
  backIcon: _vueTypes.default.any,
  prefixCls: String,
  title: _vueTypes.default.any,
  subTitle: _vueTypes.default.any,
  breadcrumb: _vueTypes.default.object,
  tags: _vueTypes.default.any,
  footer: _vueTypes.default.any,
  extra: _vueTypes.default.any,
  avatar: _vueTypes.default.object,
  ghost: {
    type: Boolean,
    default: undefined
  },
  onBack: Function
});
exports.pageHeaderProps = pageHeaderProps;
const PageHeader = (0, _vue.defineComponent)({
  compatConfig: {
    MODE: 3
  },
  name: 'APageHeader',
  props: pageHeaderProps(),
  // emits: ['back'],
  slots: Object,
  setup(props, _ref) {
    let {
      emit,
      slots
    } = _ref;
    const {
      prefixCls,
      direction,
      pageHeader
    } = (0, _useConfigInject.default)('page-header', props);
    const compact = (0, _vue.ref)(false);
    const isDestroyed = (0, _useDestroyed.default)();
    const onResize = _ref2 => {
      let {
        width
      } = _ref2;
      if (!isDestroyed.value) {
        compact.value = width < 768;
      }
    };
    const ghost = (0, _vue.computed)(() => {
      var _pageHeader$value;
      return props.ghost ?? ((_pageHeader$value = pageHeader.value) === null || _pageHeader$value === void 0 ? void 0 : _pageHeader$value.ghost) ?? true;
    });
    const getBackIcon = () => {
      var _slots$backIcon;
      return props.backIcon ?? ((_slots$backIcon = slots.backIcon) === null || _slots$backIcon === void 0 ? void 0 : _slots$backIcon.call(slots)) ?? (direction.value === 'rtl' ? (0, _vue.createVNode)(_ArrowRightOutlined.default, null, null) : (0, _vue.createVNode)(_ArrowLeftOutlined.default, null, null));
    };
    const renderBack = backIcon => {
      if (!backIcon || !props.onBack) {
        return null;
      }
      return (0, _vue.createVNode)(_LocaleReceiver.default, {
        "componentName": "PageHeader",
        "children": _ref3 => {
          let {
            back
          } = _ref3;
          return (0, _vue.createVNode)("div", {
            "class": `${prefixCls.value}-back`
          }, [(0, _vue.createVNode)(_transButton.default, {
            "onClick": e => {
              emit('back', e);
            },
            "class": `${prefixCls.value}-back-button`,
            "aria-label": back
          }, {
            default: () => [backIcon]
          })]);
        }
      }, null);
    };
    const renderBreadcrumb = () => {
      var _slots$breadcrumb;
      return props.breadcrumb ? (0, _vue.createVNode)(_breadcrumb.default, props.breadcrumb, null) : (_slots$breadcrumb = slots.breadcrumb) === null || _slots$breadcrumb === void 0 ? void 0 : _slots$breadcrumb.call(slots);
    };
    const renderTitle = () => {
      var _slots$title, _slots$subTitle, _slots$tags, _slots$extra, _slots$avatar;
      const {
        avatar
      } = props;
      const title = props.title ?? ((_slots$title = slots.title) === null || _slots$title === void 0 ? void 0 : _slots$title.call(slots));
      const subTitle = props.subTitle ?? ((_slots$subTitle = slots.subTitle) === null || _slots$subTitle === void 0 ? void 0 : _slots$subTitle.call(slots));
      const tags = props.tags ?? ((_slots$tags = slots.tags) === null || _slots$tags === void 0 ? void 0 : _slots$tags.call(slots));
      const extra = props.extra ?? ((_slots$extra = slots.extra) === null || _slots$extra === void 0 ? void 0 : _slots$extra.call(slots));
      const headingPrefixCls = `${prefixCls.value}-heading`;
      const hasHeading = title || subTitle || tags || extra;
      // If there is nothing, return a null
      if (!hasHeading) {
        return null;
      }
      const backIcon = getBackIcon();
      const backIconDom = renderBack(backIcon);
      const hasTitle = backIconDom || avatar || hasHeading;
      return (0, _vue.createVNode)("div", {
        "class": headingPrefixCls
      }, [hasTitle && (0, _vue.createVNode)("div", {
        "class": `${headingPrefixCls}-left`
      }, [backIconDom, avatar ? (0, _vue.createVNode)(_avatar.default, avatar, null) : (_slots$avatar = slots.avatar) === null || _slots$avatar === void 0 ? void 0 : _slots$avatar.call(slots), title && (0, _vue.createVNode)("span", {
        "class": `${headingPrefixCls}-title`,
        "title": typeof title === 'string' ? title : undefined
      }, [title]), subTitle && (0, _vue.createVNode)("span", {
        "class": `${headingPrefixCls}-sub-title`,
        "title": typeof subTitle === 'string' ? subTitle : undefined
      }, [subTitle]), tags && (0, _vue.createVNode)("span", {
        "class": `${headingPrefixCls}-tags`
      }, [tags])]), extra && (0, _vue.createVNode)("span", {
        "class": `${headingPrefixCls}-extra`
      }, [extra])]);
    };
    const renderFooter = () => {
      var _slots$footer;
      const footer = props.footer ?? (0, _propsUtil.filterEmpty)((_slots$footer = slots.footer) === null || _slots$footer === void 0 ? void 0 : _slots$footer.call(slots));
      return (0, _propsUtil.isEmptyContent)(footer) ? null : (0, _vue.createVNode)("div", {
        "class": `${prefixCls.value}-footer`
      }, [footer]);
    };
    const renderChildren = children => {
      return (0, _vue.createVNode)("div", {
        "class": `${prefixCls.value}-content`
      }, [children]);
    };
    return () => {
      var _props$breadcrumb, _slots$default;
      const hasBreadcrumb = ((_props$breadcrumb = props.breadcrumb) === null || _props$breadcrumb === void 0 ? void 0 : _props$breadcrumb.routes) || slots.breadcrumb;
      const hasFooter = props.footer || slots.footer;
      const children = (0, _propsUtil.flattenChildren)((_slots$default = slots.default) === null || _slots$default === void 0 ? void 0 : _slots$default.call(slots));
      const className = (0, _classNames.default)(prefixCls.value, {
        'has-breadcrumb': hasBreadcrumb,
        'has-footer': hasFooter,
        [`${prefixCls.value}-ghost`]: ghost.value,
        [`${prefixCls.value}-rtl`]: direction.value === 'rtl',
        [`${prefixCls.value}-compact`]: compact.value
      });
      return (0, _vue.createVNode)(_vcResizeObserver.default, {
        "onResize": onResize
      }, {
        default: () => [(0, _vue.createVNode)("div", {
          "class": className
        }, [renderBreadcrumb(), renderTitle(), children.length ? renderChildren(children) : null, renderFooter()])]
      });
    };
  }
});
var _default = exports.default = (0, _type.withInstall)(PageHeader);