"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _vue = require("vue");
var _colors = require("@ant-design/colors");
var _vcProgress = require("../vc-progress");
var _utils = require("./utils");
var _props = require("./props");
function getPercentage(_ref) {
  let {
    percent,
    success,
    successPercent
  } = _ref;
  const realSuccessPercent = (0, _utils.validProgress)((0, _utils.getSuccessPercent)({
    success,
    successPercent
  }));
  return [realSuccessPercent, (0, _utils.validProgress)((0, _utils.validProgress)(percent) - realSuccessPercent)];
}
function getStrokeColor(_ref2) {
  let {
    success = {},
    strokeColor
  } = _ref2;
  const {
    strokeColor: successColor
  } = success;
  return [successColor || _colors.presetPrimaryColors.green, strokeColor || null];
}
var _default = exports.default = (0, _vue.defineComponent)({
  compatConfig: {
    MODE: 3
  },
  name: 'Circle',
  inheritAttrs: false,
  props: (0, _props.progressProps)(),
  setup(props, _ref3) {
    let {
      slots
    } = _ref3;
    const gapDeg = (0, _vue.computed)(() => {
      // Support gapDeg = 0 when type = 'dashboard'
      if (props.gapDegree || props.gapDegree === 0) {
        return props.gapDegree;
      }
      if (props.type === 'dashboard') {
        return 75;
      }
      return undefined;
    });
    const circleStyle = (0, _vue.computed)(() => {
      const circleSize = props.width || 120;
      return {
        width: typeof circleSize === 'number' ? `${circleSize}px` : circleSize,
        height: typeof circleSize === 'number' ? `${circleSize}px` : circleSize,
        fontSize: `${circleSize * 0.15 + 6}px`
      };
    });
    const circleWidth = (0, _vue.computed)(() => props.strokeWidth || 6);
    const gapPos = (0, _vue.computed)(() => props.gapPosition || props.type === 'dashboard' && 'bottom' || 'top');
    // using className to style stroke color
    const percent = (0, _vue.computed)(() => getPercentage(props));
    const isGradient = (0, _vue.computed)(() => Object.prototype.toString.call(props.strokeColor) === '[object Object]');
    const strokeColor = (0, _vue.computed)(() => getStrokeColor({
      success: props.success,
      strokeColor: props.strokeColor
    }));
    const wrapperClassName = (0, _vue.computed)(() => ({
      [`${props.prefixCls}-inner`]: true,
      [`${props.prefixCls}-circle-gradient`]: isGradient.value
    }));
    return () => {
      var _slots$default;
      return (0, _vue.createVNode)("div", {
        "class": wrapperClassName.value,
        "style": circleStyle.value
      }, [(0, _vue.createVNode)(_vcProgress.Circle, {
        "percent": percent.value,
        "strokeWidth": circleWidth.value,
        "trailWidth": circleWidth.value,
        "strokeColor": strokeColor.value,
        "strokeLinecap": props.strokeLinecap,
        "trailColor": props.trailColor,
        "prefixCls": props.prefixCls,
        "gapDegree": gapDeg.value,
        "gapPosition": gapPos.value
      }, null), (_slots$default = slots.default) === null || _slots$default === void 0 ? void 0 : _slots$default.call(slots)]);
    };
  }
});