"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _vue = require("vue");
var _KeyCode = _interopRequireDefault(require("../_util/KeyCode"));
var _TextArea = _interopRequireDefault(require("../input/TextArea"));
var _EnterOutlined = _interopRequireDefault(require("@ant-design/icons-vue/lib/icons/EnterOutlined"));
const editableProps = () => ({
  prefixCls: String,
  value: String,
  maxlength: Number,
  autoSize: {
    type: [Boolean, Object]
  },
  onSave: Function,
  onCancel: Function,
  onEnd: Function,
  onChange: Function,
  originContent: String,
  direction: String
});
const Editable = (0, _vue.defineComponent)({
  compatConfig: {
    MODE: 3
  },
  name: 'Editable',
  props: editableProps(),
  // emits: ['save', 'cancel', 'end', 'change'],
  setup(props, _ref) {
    let {
      emit,
      slots
    } = _ref;
    const state = (0, _vue.reactive)({
      current: props.value || '',
      lastKeyCode: undefined,
      inComposition: false,
      cancelFlag: false
    });
    (0, _vue.watch)(() => props.value, current => {
      state.current = current;
    });
    const textArea = (0, _vue.ref)();
    (0, _vue.onMounted)(() => {
      if (textArea.value) {
        var _textArea$value;
        const resizableTextArea = (_textArea$value = textArea.value) === null || _textArea$value === void 0 ? void 0 : _textArea$value.resizableTextArea;
        const innerTextArea = resizableTextArea === null || resizableTextArea === void 0 ? void 0 : resizableTextArea.textArea;
        innerTextArea.focus();
        const {
          length
        } = innerTextArea.value;
        innerTextArea.setSelectionRange(length, length);
      }
    });
    function saveTextAreaRef(node) {
      textArea.value = node;
    }
    function onChange(_ref2) {
      let {
        target: {
          value
        }
      } = _ref2;
      state.current = value.replace(/[\r\n]/g, '');
      emit('change', state.current);
    }
    function onCompositionStart() {
      state.inComposition = true;
    }
    function onCompositionEnd() {
      state.inComposition = false;
    }
    function onKeyDown(e) {
      const {
        keyCode
      } = e;
      if (keyCode === _KeyCode.default.ENTER) {
        e.preventDefault();
      }
      // We don't record keyCode when IME is using
      if (state.inComposition) return;
      state.lastKeyCode = keyCode;
    }
    function onKeyUp(e) {
      const {
        keyCode,
        ctrlKey,
        altKey,
        metaKey,
        shiftKey
      } = e;
      // Check if it's a real key
      if (state.lastKeyCode === keyCode && !state.inComposition && !ctrlKey && !altKey && !metaKey && !shiftKey) {
        if (keyCode === _KeyCode.default.ENTER) {
          confirmChange();
          emit('end');
        } else if (keyCode === _KeyCode.default.ESC) {
          state.current = props.originContent;
          emit('cancel');
        }
      }
    }
    function onBlur() {
      confirmChange();
    }
    function confirmChange() {
      emit('save', state.current.trim());
    }
    const textAreaClassName = (0, _vue.computed)(() => ({
      [`${props.prefixCls}`]: true,
      [`${props.prefixCls}-edit-content`]: true,
      [`${props.prefixCls}-rtl`]: props.direction === 'rtl'
    }));
    return () => (0, _vue.createVNode)("div", {
      "class": textAreaClassName.value
    }, [(0, _vue.createVNode)(_TextArea.default, {
      "ref": saveTextAreaRef,
      "maxlength": props.maxlength,
      "value": state.current,
      "onChange": onChange,
      "onKeydown": onKeyDown,
      "onKeyup": onKeyUp,
      "onCompositionstart": onCompositionStart,
      "onCompositionend": onCompositionEnd,
      "onBlur": onBlur,
      "rows": 1,
      "autoSize": props.autoSize === undefined || props.autoSize
    }, null), slots.enterIcon ? slots.enterIcon({
      className: `${props.prefixCls}-edit-content-confirm`
    }) : (0, _vue.createVNode)(_EnterOutlined.default, {
      "class": `${props.prefixCls}-edit-content-confirm`
    }, null)]);
  }
});
var _default = exports.default = Editable;