"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _vue = require("vue");
var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));
var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));
var _classNames = _interopRequireDefault(require("../../_util/classNames"));
var _getScrollBarSize = _interopRequireDefault(require("../../_util/getScrollBarSize"));
var _KeyCode = _interopRequireDefault(require("../../_util/KeyCode"));
var _omit = _interopRequireDefault(require("../../_util/omit"));
var _supportsPassive = _interopRequireDefault(require("../../_util/supportsPassive"));
var _IDrawerPropTypes = require("./IDrawerPropTypes");
var _utils = require("./utils");
const _excluded = ["width", "height", "open", "prefixCls", "placement", "level", "levelMove", "ease", "duration", "getContainer", "onChange", "afterVisibleChange", "showMask", "maskClosable", "maskStyle", "keyboard", "getOpenCount", "scrollLocker", "contentWrapperStyle", "style", "class"];
const currentDrawer = {};
const DrawerChild = (0, _vue.defineComponent)({
  compatConfig: {
    MODE: 3
  },
  inheritAttrs: false,
  props: (0, _IDrawerPropTypes.drawerChildProps)(),
  emits: ['close', 'handleClick', 'change'],
  setup(props, _ref) {
    let {
      emit,
      slots
    } = _ref;
    const state = (0, _vue.reactive)({
      startPos: {
        x: null,
        y: null
      }
    });
    let timeout;
    const contentWrapper = (0, _vue.ref)();
    const dom = (0, _vue.ref)();
    const maskDom = (0, _vue.ref)();
    const handlerDom = (0, _vue.ref)();
    const contentDom = (0, _vue.ref)();
    let levelDom = [];
    const drawerId = `drawer_id_${Number((Date.now() + Math.random()).toString().replace('.', Math.round(Math.random() * 9).toString())).toString(16)}`;
    const passive = !_utils.windowIsUndefined && _supportsPassive.default ? {
      passive: false
    } : false;
    (0, _vue.onMounted)(() => {
      (0, _vue.nextTick)(() => {
        const {
          open,
          getContainer,
          showMask,
          autofocus
        } = props;
        const container = getContainer === null || getContainer === void 0 ? void 0 : getContainer();
        getLevelDom(props);
        if (open) {
          if (container && container.parentNode === document.body) {
            currentDrawer[drawerId] = open;
          }
          // 默认打开状态时推出 level;
          openLevelTransition();
          (0, _vue.nextTick)(() => {
            if (autofocus) {
              domFocus();
            }
          });
          if (showMask) {
            var _props$scrollLocker;
            (_props$scrollLocker = props.scrollLocker) === null || _props$scrollLocker === void 0 ? void 0 : _props$scrollLocker.lock();
          }
        }
      });
    });
    (0, _vue.watch)(() => props.level, () => {
      getLevelDom(props);
    }, {
      flush: 'post'
    });
    (0, _vue.watch)(() => props.open, () => {
      const {
        open,
        getContainer,
        scrollLocker,
        showMask,
        autofocus
      } = props;
      const container = getContainer === null || getContainer === void 0 ? void 0 : getContainer();
      if (container && container.parentNode === document.body) {
        currentDrawer[drawerId] = !!open;
      }
      openLevelTransition();
      if (open) {
        if (autofocus) {
          domFocus();
        }
        if (showMask) {
          scrollLocker === null || scrollLocker === void 0 ? void 0 : scrollLocker.lock();
        }
      } else {
        scrollLocker === null || scrollLocker === void 0 ? void 0 : scrollLocker.unLock();
      }
    }, {
      flush: 'post'
    });
    (0, _vue.onUnmounted)(() => {
      var _props$scrollLocker2;
      const {
        open
      } = props;
      delete currentDrawer[drawerId];
      if (open) {
        setLevelTransform(false);
        document.body.style.touchAction = '';
      }
      (_props$scrollLocker2 = props.scrollLocker) === null || _props$scrollLocker2 === void 0 ? void 0 : _props$scrollLocker2.unLock();
    });
    (0, _vue.watch)(() => props.placement, val => {
      if (val) {
        // test 的 bug, 有动画过场，删除 dom
        contentDom.value = null;
      }
    });
    const domFocus = () => {
      var _dom$value, _dom$value$focus;
      (_dom$value = dom.value) === null || _dom$value === void 0 ? void 0 : (_dom$value$focus = _dom$value.focus) === null || _dom$value$focus === void 0 ? void 0 : _dom$value$focus.call(_dom$value);
    };
    const removeStartHandler = e => {
      if (e.touches.length > 1) {
        return;
      }
      state.startPos = {
        x: e.touches[0].clientX,
        y: e.touches[0].clientY
      };
    };
    const removeMoveHandler = e => {
      if (e.changedTouches.length > 1) {
        return;
      }
      const currentTarget = e.currentTarget;
      const differX = e.changedTouches[0].clientX - state.startPos.x;
      const differY = e.changedTouches[0].clientY - state.startPos.y;
      if ((currentTarget === maskDom.value || currentTarget === handlerDom.value || currentTarget === contentDom.value && (0, _utils.getTouchParentScroll)(currentTarget, e.target, differX, differY)) && e.cancelable) {
        e.preventDefault();
      }
    };
    const transitionEnd = e => {
      const dom = e.target;
      (0, _utils.removeEventListener)(dom, _utils.transitionEndFun, transitionEnd);
      dom.style.transition = '';
    };
    const onClose = e => {
      emit('close', e);
    };
    const onKeyDown = e => {
      if (e.keyCode === _KeyCode.default.ESC) {
        e.stopPropagation();
        onClose(e);
      }
    };
    const onWrapperTransitionEnd = e => {
      const {
        open,
        afterVisibleChange
      } = props;
      if (e.target === contentWrapper.value && e.propertyName.match(/transform$/)) {
        dom.value.style.transition = '';
        if (!open && getCurrentDrawerSome()) {
          document.body.style.overflowX = '';
          if (maskDom.value) {
            maskDom.value.style.left = '';
            maskDom.value.style.width = '';
          }
        }
        if (afterVisibleChange) {
          afterVisibleChange(!!open);
        }
      }
    };
    const horizontalBoolAndPlacementName = (0, _vue.computed)(() => {
      const {
        placement
      } = props;
      const isHorizontal = placement === 'left' || placement === 'right';
      const placementName = `translate${isHorizontal ? 'X' : 'Y'}`;
      return {
        isHorizontal,
        placementName
      };
    });
    const openLevelTransition = () => {
      const {
        open,
        width,
        height
      } = props;
      const {
        isHorizontal,
        placementName
      } = horizontalBoolAndPlacementName.value;
      const contentValue = contentDom.value ? contentDom.value.getBoundingClientRect()[isHorizontal ? 'width' : 'height'] : 0;
      const value = (isHorizontal ? width : height) || contentValue;
      setLevelAndScrolling(open, placementName, value);
    };
    const setLevelTransform = (open, placementName, value, right) => {
      const {
        placement,
        levelMove,
        duration,
        ease,
        showMask
      } = props;
      // router 切换时可能会导至页面失去滚动条，所以需要时时获取。
      levelDom.forEach(dom => {
        dom.style.transition = `transform ${duration} ${ease}`;
        (0, _utils.addEventListener)(dom, _utils.transitionEndFun, transitionEnd);
        let levelValue = open ? value : 0;
        if (levelMove) {
          const $levelMove = (0, _utils.transformArguments)(levelMove, {
            target: dom,
            open
          });
          levelValue = open ? $levelMove[0] : $levelMove[1] || 0;
        }
        const $value = typeof levelValue === 'number' ? `${levelValue}px` : levelValue;
        let placementPos = placement === 'left' || placement === 'top' ? $value : `-${$value}`;
        placementPos = showMask && placement === 'right' && right ? `calc(${placementPos} + ${right}px)` : placementPos;
        dom.style.transform = levelValue ? `${placementName}(${placementPos})` : '';
      });
    };
    const setLevelAndScrolling = (open, placementName, value) => {
      if (!_utils.windowIsUndefined) {
        const right = document.body.scrollHeight > (window.innerHeight || document.documentElement.clientHeight) && window.innerWidth > document.body.offsetWidth ? (0, _getScrollBarSize.default)(true) : 0;
        setLevelTransform(open, placementName, value, right);
        toggleScrollingToDrawerAndBody(right);
      }
      emit('change', open);
    };
    const toggleScrollingToDrawerAndBody = right => {
      const {
        getContainer,
        showMask,
        open
      } = props;
      const container = getContainer === null || getContainer === void 0 ? void 0 : getContainer();
      // 处理 body 滚动
      if (container && container.parentNode === document.body && showMask) {
        const eventArray = ['touchstart'];
        const domArray = [document.body, maskDom.value, handlerDom.value, contentDom.value];
        if (open && document.body.style.overflow !== 'hidden') {
          if (right) {
            addScrollingEffect(right);
          }
          document.body.style.touchAction = 'none';
          // 手机禁滚
          domArray.forEach((item, i) => {
            if (!item) {
              return;
            }
            (0, _utils.addEventListener)(item, eventArray[i] || 'touchmove', i ? removeMoveHandler : removeStartHandler, passive);
          });
        } else if (getCurrentDrawerSome()) {
          document.body.style.touchAction = '';
          if (right) {
            remScrollingEffect(right);
          }
          // 恢复事件
          domArray.forEach((item, i) => {
            if (!item) {
              return;
            }
            (0, _utils.removeEventListener)(item, eventArray[i] || 'touchmove', i ? removeMoveHandler : removeStartHandler, passive);
          });
        }
      }
    };
    const addScrollingEffect = right => {
      const {
        placement,
        duration,
        ease
      } = props;
      const widthTransition = `width ${duration} ${ease}`;
      const transformTransition = `transform ${duration} ${ease}`;
      dom.value.style.transition = 'none';
      switch (placement) {
        case 'right':
          dom.value.style.transform = `translateX(-${right}px)`;
          break;
        case 'top':
        case 'bottom':
          dom.value.style.width = `calc(100% - ${right}px)`;
          dom.value.style.transform = 'translateZ(0)';
          break;
        default:
          break;
      }
      clearTimeout(timeout);
      timeout = setTimeout(() => {
        if (dom.value) {
          dom.value.style.transition = `${transformTransition},${widthTransition}`;
          dom.value.style.width = '';
          dom.value.style.transform = '';
        }
      });
    };
    const remScrollingEffect = right => {
      const {
        placement,
        duration,
        ease
      } = props;
      dom.value.style.transition = 'none';
      let heightTransition;
      let widthTransition = `width ${duration} ${ease}`;
      const transformTransition = `transform ${duration} ${ease}`;
      switch (placement) {
        case 'left':
          {
            dom.value.style.width = '100%';
            widthTransition = `width 0s ${ease} ${duration}`;
            break;
          }
        case 'right':
          {
            dom.value.style.transform = `translateX(${right}px)`;
            dom.value.style.width = '100%';
            widthTransition = `width 0s ${ease} ${duration}`;
            if (maskDom.value) {
              maskDom.value.style.left = `-${right}px`;
              maskDom.value.style.width = `calc(100% + ${right}px)`;
            }
            break;
          }
        case 'top':
        case 'bottom':
          {
            dom.value.style.width = `calc(100% + ${right}px)`;
            dom.value.style.height = '100%';
            dom.value.style.transform = 'translateZ(0)';
            heightTransition = `height 0s ${ease} ${duration}`;
            break;
          }
        default:
          break;
      }
      clearTimeout(timeout);
      timeout = setTimeout(() => {
        if (dom.value) {
          dom.value.style.transition = `${transformTransition},${heightTransition ? `${heightTransition},` : ''}${widthTransition}`;
          dom.value.style.transform = '';
          dom.value.style.width = '';
          dom.value.style.height = '';
        }
      });
    };
    const getCurrentDrawerSome = () => !Object.keys(currentDrawer).some(key => currentDrawer[key]);
    const getLevelDom = _ref2 => {
      let {
        level,
        getContainer
      } = _ref2;
      if (_utils.windowIsUndefined) {
        return;
      }
      const container = getContainer === null || getContainer === void 0 ? void 0 : getContainer();
      const parent = container ? container.parentNode : null;
      levelDom = [];
      if (level === 'all') {
        const children = parent ? Array.prototype.slice.call(parent.children) : [];
        children.forEach(child => {
          if (child.nodeName !== 'SCRIPT' && child.nodeName !== 'STYLE' && child.nodeName !== 'LINK' && child !== container) {
            levelDom.push(child);
          }
        });
      } else if (level) {
        (0, _utils.dataToArray)(level).forEach(key => {
          document.querySelectorAll(key).forEach(item => {
            levelDom.push(item);
          });
        });
      }
    };
    const onHandleClick = e => {
      emit('handleClick', e);
    };
    const canOpen = (0, _vue.ref)(false);
    (0, _vue.watch)(dom, () => {
      (0, _vue.nextTick)(() => {
        canOpen.value = true;
      });
    });
    return () => {
      var _slots$default, _slots$handler;
      const {
          width,
          height,
          open: $open,
          prefixCls,
          placement,
          level,
          levelMove,
          ease,
          duration,
          getContainer,
          onChange,
          afterVisibleChange,
          showMask,
          maskClosable,
          maskStyle,
          keyboard,
          getOpenCount,
          scrollLocker,
          contentWrapperStyle,
          style,
          class: className
        } = props,
        otherProps = (0, _objectWithoutProperties2.default)(props, _excluded);
      // 首次渲染都将是关闭状态。
      const open = $open && canOpen.value;
      const wrapperClassName = (0, _classNames.default)(prefixCls, {
        [`${prefixCls}-${placement}`]: true,
        [`${prefixCls}-open`]: open,
        [className]: !!className,
        'no-mask': !showMask
      });
      const {
        placementName
      } = horizontalBoolAndPlacementName.value;
      // 百分比与像素动画不同步，第一次打用后全用像素动画。
      // const defaultValue = !this.contentDom || !level ? '100%' : `${value}px`;
      const placementPos = placement === 'left' || placement === 'top' ? '-100%' : '100%';
      const transform = open ? '' : `${placementName}(${placementPos})`;
      return (0, _vue.createVNode)("div", (0, _objectSpread2.default)((0, _objectSpread2.default)({}, (0, _omit.default)(otherProps, ['switchScrollingEffect', 'autofocus'])), {}, {
        "tabindex": -1,
        "class": wrapperClassName,
        "style": style,
        "ref": dom,
        "onKeydown": open && keyboard ? onKeyDown : undefined,
        "onTransitionend": onWrapperTransitionEnd
      }), [showMask && (0, _vue.createVNode)("div", {
        "class": `${prefixCls}-mask`,
        "onClick": maskClosable ? onClose : undefined,
        "style": maskStyle,
        "ref": maskDom
      }, null), (0, _vue.createVNode)("div", {
        "class": `${prefixCls}-content-wrapper`,
        "style": (0, _objectSpread2.default)({
          transform,
          msTransform: transform,
          width: (0, _utils.isNumeric)(width) ? `${width}px` : width,
          height: (0, _utils.isNumeric)(height) ? `${height}px` : height
        }, contentWrapperStyle),
        "ref": contentWrapper
      }, [(0, _vue.createVNode)("div", {
        "class": `${prefixCls}-content`,
        "ref": contentDom
      }, [(_slots$default = slots.default) === null || _slots$default === void 0 ? void 0 : _slots$default.call(slots)]), slots.handler ? (0, _vue.createVNode)("div", {
        "onClick": onHandleClick,
        "ref": handlerDom
      }, [(_slots$handler = slots.handler) === null || _slots$handler === void 0 ? void 0 : _slots$handler.call(slots)]) : null])]);
    };
  }
});
var _default = exports.default = DrawerChild;